import { jsx as _jsx } from "@emotion/react/jsx-runtime";
import { useState, useEffect, useContext, createContext } from "react";
import { Provider as ReduxProvider } from "react-redux";
import { ServiceManager, ServerConnection } from '@jupyterlab/services';
import { getJupyterServerHttpUrl } from './JupyterConfig';
import { requestAPI } from './JupyterHandlers';
import { startLiteServer } from './../jupyter/lite/LiteServer';
/**
 * The instance for the Jupyter context.
 */
export const JupyterContext = createContext(undefined);
export const useJupyter = () => {
    const context = useContext(JupyterContext);
    if (!context)
        throw new Error("useContext must be inside a provider with a value.");
    return context;
};
/**
 * The type for the Jupyter context consumer.
 */
export const JupyterContextConsumer = JupyterContext.Consumer;
/**
 * The type for the Jupyter context provider.
 */
const JupyterProvider = JupyterContext.Provider;
/**
 * Utiliy method to ensure the Jupyter context is authenticated
 * with the Jupyter server.
 */
export const ensureJupyterAuth = () => {
    return requestAPI('api', '', {
        credentials: 'include',
        mode: 'cors',
    })
        .then(data => {
        return true;
    })
        .catch(reason => {
        console.log('The jupyter server API has failed with reason', reason);
        return false;
    });
};
const headers = new Headers({
    "Cache-Control": "no-cache, no-store, must-revalidate",
    "Pragma": "no-cache",
    "Expires": "0",
});
/**
 * The Jupyter context provider.
 */
export const JupyterContextProvider = ({ children, lite, variant, baseUrl, wsUrl, injectableStore }) => {
    const [_, setVariant] = useState('default');
    const [serverSettings, setServerSettings] = useState();
    const [serviceManager, setServiceManager] = useState();
    useEffect(() => {
        if (lite) {
            startLiteServer().then((serviceManager) => {
                setServiceManager(serviceManager);
            });
        }
        else {
            ensureJupyterAuth().then(isAuth => {
                if (isAuth) {
                    const serverSettings = ServerConnection.makeSettings({
                        baseUrl,
                        wsUrl,
                        appendToken: true,
                        init: {
                            credentials: "include",
                            mode: 'cors',
                            headers,
                        }
                    });
                    setServerSettings(serverSettings);
                    const serviceManager = new ServiceManager({ serverSettings });
                    setServiceManager(serviceManager);
                }
                else {
                    const loginUrl = getJupyterServerHttpUrl() + '/login?next=' + window.location;
                    console.warn('Redirecting to jupyter login url', loginUrl);
                    window.location.replace(loginUrl);
                }
            });
        }
        setVariant(variant);
    }, [lite, variant]);
    return (_jsx(ReduxProvider, Object.assign({ store: injectableStore }, { children: _jsx(JupyterProvider, Object.assign({ value: {
                lite,
                serverSettings,
                serviceManager,
                variant,
                setVariant,
                baseUrl,
                wsUrl,
                injectableStore,
            } }, { children: children }), void 0) }), void 0));
};
//# sourceMappingURL=JupyterContext.js.map